@@ -3,7 +3,7 @@ BEGIN {
   $App::Pocoirc::AUTHORITY = 'cpan:HINRIK';
 }
 BEGIN {
-  $App::Pocoirc::VERSION = '0.04';
+  $App::Pocoirc::VERSION = '0.05';
 }
 
 use strict;
@@ -366,6 +366,7 @@ sub _exception {
     chomp $ex->{error_str};
     warn "Event $ex->{event} in session "
         .$ex->{dest_session}->ID." raised exception:\n  $ex->{error_str}\n";
+    $self->_shutdown('Caught exception');
     $kernel->sig_handled();
     return;
 }
@@ -374,18 +375,27 @@ sub _exit {
     my ($kernel, $self) = @_[KERNEL, OBJECT];
 
     $self->_status('Caught interrupt signal, exiting...');
-    for my $irc (@{ $self->{ircs} }) {
-        my ($network, $obj) = @$irc;
-        $obj->connected
-            ? $obj->yield(quit => 'Caught interrupt')
-            : $obj->shutdown();
-    }
-    $self->{shutdown} = 1;
-
+    $self->_shutdown('Caught interrupt');
     $kernel->sig_handled();
     return;
 }
 
+sub _shutdown {
+    my ($self, $reason) = @_;
+
+    if (!$self->{shutdown}) {
+        for my $irc (@{ $self->{ircs} }) {
+            my ($network, $obj) = @$irc;
+            $obj->connected
+                ? $obj->yield(quit => $reason)
+                : $obj->shutdown();
+        }
+        $self->{shutdown} = 1;
+    }
+
+    return;
+}
+
 1;
 
 =encoding utf8
